@@ -1,33 +1,32 @@
-"""
-    sphinx.pycode
-    ~~~~~~~~~~~~~
+"""Utilities parsing and analyzing Python code."""
 
-    Utilities parsing and analyzing Python code.
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-import re
 import tokenize
-import warnings
 from importlib import import_module
-from io import StringIO
 from os import path
-from typing import Any, Dict, IO, List, Tuple, Optional
-from zipfile import ZipFile
+from typing import TYPE_CHECKING, Any
 
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.errors import PycodeError
 from sphinx.pycode.parser import Parser
 
+if TYPE_CHECKING:
+    from inspect import Signature
+
 
 class ModuleAnalyzer:
+    annotations: dict[tuple[str, str], str]
+    attr_docs: dict[tuple[str, str], list[str]]
+    finals: list[str]
+    overloads: dict[str, list[Signature]]
+    tagorder: dict[str, int]
+    tags: dict[str, tuple[str, int, int]]
+
     # cache for analyzer objects -- caches both by module and file name
-    cache = {}  # type: Dict[Tuple[str, str], Any]
+    cache: dict[tuple[str, str], Any] = {}
 
     @staticmethod
-    def get_module_source(modname: str) -> Tuple[Optional[str], Optional[str]]:
+    def get_module_source(modname: str) -> tuple[str | None, str | None]:
         """Try to find the source code for a module.
 
         Returns ('filename', 'source'). One of it can be None if
@@ -36,7 +35,7 @@ class ModuleAnalyzer:
         try:
             mod = import_module(modname)
         except Exception as err:
-            raise PycodeError('error importing %r' % modname, err)
+            raise PycodeError('error importing %r' % modname, err) from err
         loader = getattr(mod, '__loader__', None)
         filename = getattr(mod, '__file__', None)
         if loader and getattr(loader, 'get_source', None):
@@ -53,7 +52,9 @@ class ModuleAnalyzer:
             try:
                 filename = loader.get_filename(modname)
             except ImportError as err:
-                raise PycodeError('error getting filename for %r' % modname, err)
+                raise PycodeError(
+                    'error getting filename for %r' % modname, err
+                ) from err
         if filename is None:
             # all methods for getting filename failed, so raise...
             raise PycodeError('no source found for module %r' % modname)
@@ -64,49 +65,37 @@ class ModuleAnalyzer:
                 filename += 'w'
         elif not filename.lower().endswith(('.py', '.pyw')):
             raise PycodeError('source is not a .py file: %r' % filename)
-        elif ('.egg' + path.sep) in filename:
-            pat = '(?<=\\.egg)' + re.escape(path.sep)
-            eggpath, _ = re.split(pat, filename, 1)
-            if path.isfile(eggpath):
-                return filename, None
 
         if not path.isfile(filename):
             raise PycodeError('source file is not present: %r' % filename)
         return filename, None
 
     @classmethod
-    def for_string(cls, string: str, modname: str, srcname: str = '<string>'
-                   ) -> "ModuleAnalyzer":
-        return cls(StringIO(string), modname, srcname, decoded=True)
+    def for_string(
+        cls: type[ModuleAnalyzer],
+        string: str,
+        modname: str,
+        srcname: str = '<string>',
+    ) -> ModuleAnalyzer:
+        return cls(string, modname, srcname)
 
     @classmethod
-    def for_file(cls, filename: str, modname: str) -> "ModuleAnalyzer":
+    def for_file(
+        cls: type[ModuleAnalyzer], filename: str, modname: str
+    ) -> ModuleAnalyzer:
         if ('file', filename) in cls.cache:
             return cls.cache['file', filename]
         try:
             with tokenize.open(filename) as f:
-                obj = cls(f, modname, filename, decoded=True)
-                cls.cache['file', filename] = obj
+                string = f.read()
+            obj = cls(string, modname, filename)
+            cls.cache['file', filename] = obj
         except Exception as err:
-            if '.egg' + path.sep in filename:
-                obj = cls.cache['file', filename] = cls.for_egg(filename, modname)
-            else:
-                raise PycodeError('error opening %r' % filename, err)
+            raise PycodeError('error opening %r' % filename, err) from err
         return obj
 
     @classmethod
-    def for_egg(cls, filename: str, modname: str) -> "ModuleAnalyzer":
-        SEP = re.escape(path.sep)
-        eggpath, relpath = re.split('(?<=\\.egg)' + SEP, filename)
-        try:
-            with ZipFile(eggpath) as egg:
-                code = egg.read(relpath).decode()
-                return cls.for_string(code, modname, filename)
-        except Exception as exc:
-            raise PycodeError('error opening %r' % filename, exc)
-
-    @classmethod
-    def for_module(cls, modname: str) -> "ModuleAnalyzer":
+    def for_module(cls: type[ModuleAnalyzer], modname: str) -> ModuleAnalyzer:
         if ('module', modname) in cls.cache:
             entry = cls.cache['module', modname]
             if isinstance(entry, PycodeError):
@@ -125,63 +114,47 @@ class ModuleAnalyzer:
         cls.cache['module', modname] = obj
         return obj
 
-    def __init__(self, source: IO, modname: str, srcname: str, decoded: bool = False) -> None:
+    def __init__(self, source: str, modname: str, srcname: str) -> None:
         self.modname = modname  # name of the module
         self.srcname = srcname  # name of the source file
 
         # cache the source code as well
-        pos = source.tell()
-        if not decoded:
-            warnings.warn('decode option for ModuleAnalyzer is deprecated.',
-                          RemovedInSphinx40Warning)
-            self._encoding, _ = tokenize.detect_encoding(source.readline)
-            source.seek(pos)
-            self.code = source.read().decode(self._encoding)
-        else:
-            self._encoding = None
-            self.code = source.read()
+        self.code = source
 
-        # will be filled by parse()
-        self.annotations = None  # type: Dict[Tuple[str, str], str]
-        self.attr_docs = None    # type: Dict[Tuple[str, str], List[str]]
-        self.tagorder = None     # type: Dict[str, int]
-        self.tags = None         # type: Dict[str, Tuple[str, int, int]]
+        self._analyzed = False
+
+    def analyze(self) -> None:
+        """Analyze the source code."""
+        if self._analyzed:
+            return
 
-    def parse(self) -> None:
-        """Parse the source code."""
         try:
-            parser = Parser(self.code, self._encoding)
+            parser = Parser(self.code)
             parser.parse()
 
             self.attr_docs = {}
-            for (scope, comment) in parser.comments.items():
+            for scope, comment in parser.comments.items():
                 if comment:
-                    self.attr_docs[scope] = comment.splitlines() + ['']
+                    self.attr_docs[scope] = [*comment.splitlines(), '']
                 else:
                     self.attr_docs[scope] = ['']
 
             self.annotations = parser.annotations
+            self.finals = parser.finals
+            self.overloads = parser.overloads
             self.tags = parser.definitions
             self.tagorder = parser.deforders
+            self._analyzed = True
         except Exception as exc:
-            raise PycodeError('parsing %r failed: %r' % (self.srcname, exc))
+            msg = f'parsing {self.srcname!r} failed: {exc!r}'
+            raise PycodeError(msg) from exc
 
-    def find_attr_docs(self) -> Dict[Tuple[str, str], List[str]]:
+    def find_attr_docs(self) -> dict[tuple[str, str], list[str]]:
         """Find class and module-level attributes and their documentation."""
-        if self.attr_docs is None:
-            self.parse()
-
+        self.analyze()
         return self.attr_docs
 
-    def find_tags(self) -> Dict[str, Tuple[str, int, int]]:
+    def find_tags(self) -> dict[str, tuple[str, int, int]]:
         """Find class, function and method definitions and their location."""
-        if self.tags is None:
-            self.parse()
-
+        self.analyze()
         return self.tags
-
-    @property
-    def encoding(self) -> str:
-        warnings.warn('ModuleAnalyzer.encoding is deprecated.',
-                      RemovedInSphinx40Warning)
-        return self._encoding
